@@ -85,7 +85,6 @@ static char *ms_errorCodes[MS_NUMERRORCODES] = {"",
     "OpenGL renderer error.",
     "Renderer error."
                                                };
-
 #ifndef USE_THREAD
 
 errorObj *msGetErrorObj()
@@ -515,9 +514,6 @@ char *msGetVersion()
 #if (defined USE_GD_JPEG || defined USE_JPEG)
   strcat(version, " OUTPUT=JPEG");
 #endif
-#ifdef USE_PDF
-  strcat(version, " OUTPUT=PDF");
-#endif
 #ifdef USE_KML
   strcat(version, " OUTPUT=KML");
 #endif
@@ -532,8 +528,13 @@ char *msGetVersion()
 #ifdef USE_CAIRO
   strcat(version, " SUPPORTS=CAIRO");
 #endif
-#ifdef USE_SVG_CAIRO
+#if defined(USE_SVG_CAIRO) || defined(USE_RSVG)
   strcat(version, " SUPPORTS=SVG_SYMBOLS");
+  #ifdef USE_SVG_CAIRO
+    strcat(version, " SUPPORTS=SVGCAIRO");
+  #else
+    strcat(version, " SUPPORTS=RSVG");
+  #endif
 #endif
 #ifdef USE_OGL
   strcat(version, " SUPPORTS=OPENGL");
@@ -577,12 +578,6 @@ char *msGetVersion()
 #ifdef USE_POINT_Z_M
   strcat(version, " SUPPORTS=POINT_Z_M");
 #endif
-#ifdef USE_TIFF
-  strcat(version, " INPUT=TIFF");
-#endif
-#ifdef USE_EPPL
-  strcat(version, " INPUT=EPPL7");
-#endif
 #ifdef USE_JPEG
   strcat(version, " INPUT=JPEG");
 #endif
