@@ -1,24 +1,23 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+
+/* A Bison parser, made by GNU Bison 2.4.1.  */
 
 /* Skeleton interface for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,10 +28,11 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+
 /* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
@@ -70,67 +70,41 @@
      LENGTH = 286,
      AREA = 287,
      TOSTRING = 288,
-     DIFFERENCE = 289,
-     YYBUFFER = 290,
-     NEG = 291
+     SMOOTHSIA = 289,
+     GENERALIZE = 290,
+     SIMPLIFYPT = 291,
+     SIMPLIFY = 292,
+     DIFFERENCE = 293,
+     YYBUFFER = 294,
+     NEG = 295
    };
 #endif
-/* Tokens.  */
-#define NUMBER 258
-#define STRING 259
-#define TIME 260
-#define SHAPE 261
-#define OR 262
-#define AND 263
-#define NOT 264
-#define IRE 265
-#define IEQ 266
-#define IN 267
-#define GE 268
-#define LE 269
-#define GT 270
-#define LT 271
-#define NE 272
-#define EQ 273
-#define RE 274
-#define DWITHIN 275
-#define BEYOND 276
-#define CONTAINS 277
-#define WITHIN 278
-#define CROSSES 279
-#define OVERLAPS 280
-#define TOUCHES 281
-#define DISJOINT 282
-#define INTERSECTS 283
-#define ROUND 284
-#define COMMIFY 285
-#define LENGTH 286
-#define AREA 287
-#define TOSTRING 288
-#define DIFFERENCE 289
-#define YYBUFFER 290
-#define NEG 291
-
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 30 "mapparser.y"
 {
+
+/* Line 1676 of yacc.c  */
+#line 30 "/home/even/mapserver/git/mapserver/mapparser.y"
+
   double dblval;
   int intval;  
   char *strval;  
   struct tm tmval;
   shapeObj *shpval;
-}
-/* Line 1529 of yacc.c.  */
-#line 129 "mapparser.h"
-	YYSTYPE;
+
+
+
+/* Line 1676 of yacc.c  */
+#line 102 "/home/even/mapserver/git/mapserver/mapparser.h"
+} YYSTYPE;
+# define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
 
+
