@@ -16,15 +16,6 @@ public class DrawMap {
 
   public static void main(String[] args) {
     if (args.length != 2) usage();
-	try
-	{
-	  System.loadLibrary("mapscript");
-	}
-	catch(UnsatisfiedLinkError ule)
-	{
-	  System.err.println(ule);
-	  System.exit(-1);
-	}    
    
     mapObj map = new mapObj(args[0]);
     //map.getImagecolor().setRGB(153, 153, 204);
@@ -36,6 +27,8 @@ public class DrawMap {
     int i=0;
     //for (i=0; i<100; i++) {
     	imageObj img = map.draw();
+    	System.out.println("Image size is: "+img.getSize());
+    	System.out.println("Image size from getBytes is: "+img.getBytes().length);
     	System.out.println(i+") the map will be drawn to:"+args[1]);
     	img.save(args[1], map);
     //}
