@@ -165,7 +165,7 @@ PHP_METHOD(classObj, __construct)
 PHP_METHOD(classObj, __get)
 {
   char *property;
-  long property_len;
+  long property_len = 0;
   zval *zobj = getThis();
   php_class_object *php_class;
 
@@ -201,7 +201,7 @@ PHP_METHOD(classObj, __get)
 PHP_METHOD(classObj, __set)
 {
   char *property;
-  long property_len;
+  long property_len = 0;
   zval *value;
   zval *zobj = getThis();
   php_class_object *php_class;
@@ -338,7 +338,7 @@ PHP_METHOD(classObj, getLabel)
 PHP_METHOD(classObj, updateFromString)
 {
   char *snippet;
-  long snippet_len;
+  long snippet_len = 0;
   zval *zobj = getThis();
   php_class_object *php_class;
   int status = MS_FAILURE;
@@ -365,12 +365,39 @@ PHP_METHOD(classObj, updateFromString)
 }
 /* }}} */
 
+/* {{{ proto string convertToString()
+   Convert the class object to string. */
+PHP_METHOD(classObj, convertToString)
+{
+  zval *zobj = getThis();
+  php_class_object *php_class;
+  char *value = NULL;
+
+  PHP_MAPSCRIPT_ERROR_HANDLING(TRUE);
+  if (zend_parse_parameters_none() == FAILURE) {
+    PHP_MAPSCRIPT_RESTORE_ERRORS(TRUE);
+    return;
+  }
+  PHP_MAPSCRIPT_RESTORE_ERRORS(TRUE);
+
+  php_class = (php_class_object *) zend_object_store_get_object(zobj TSRMLS_CC);
+
+  value =  classObj_convertToString(php_class->class);
+
+  if (value == NULL)
+    RETURN_STRING("", 1);
+
+  RETVAL_STRING(value, 1);
+  free(value);
+}
+/* }}} */
+
 /* {{{ proto int setExpression(string exression)
    Set the expression string for a class object. */
 PHP_METHOD(classObj, setExpression)
 {
   char *expression;
-  long expression_len;
+  long expression_len = 0;
   zval *zobj = getThis();
   php_class_object *php_class;
   int status = MS_FAILURE;
@@ -429,7 +456,7 @@ PHP_METHOD(classObj, getExpressionString)
 PHP_METHOD(classObj, setText)
 {
   char *text;
-  long text_len;
+  long text_len = 0;
   zval *zobj = getThis();
   php_class_object *php_class;
   php_layer_object *php_layer;
@@ -855,6 +882,7 @@ zend_function_entry class_functions[] = {
   PHP_ME(classObj, removeLabel, class_removeLabel_args, ZEND_ACC_PUBLIC)
   PHP_ME(classObj, getLabel, class_getLabel_args, ZEND_ACC_PUBLIC)
   PHP_ME(classObj, updateFromString, class_updateFromString_args, ZEND_ACC_PUBLIC)
+  PHP_ME(classObj, convertToString, NULL, ZEND_ACC_PUBLIC)
   PHP_ME(classObj, setExpression, class_setExpression_args, ZEND_ACC_PUBLIC)
   PHP_ME(classObj, getExpressionString, NULL, ZEND_ACC_PUBLIC)
   PHP_ME(classObj, setText, class_setText_args, ZEND_ACC_PUBLIC)
