@@ -118,7 +118,7 @@ PHP_METHOD(labelObj, __construct)
 PHP_METHOD(labelObj, __get)
 {
   char *property;
-  long property_len;
+  long property_len = 0;
   zval *zobj = getThis();
   php_label_object *php_label;
 
@@ -175,7 +175,7 @@ PHP_METHOD(labelObj, __get)
 PHP_METHOD(labelObj, __set)
 {
   char *property;
-  long property_len;
+  long property_len = 0;
   zval *value;
   zval *zobj = getThis();
   php_label_object *php_label;
@@ -239,7 +239,7 @@ PHP_METHOD(labelObj, updateFromString)
 {
   zval *zobj = getThis();
   char *snippet;
-  long snippet_len;
+  long snippet_len = 0;
   int status = MS_FAILURE;
   php_label_object *php_label;
 
@@ -262,13 +262,40 @@ PHP_METHOD(labelObj, updateFromString)
 }
 /* }}} */
 
+/* {{{ proto string convertToString()
+   Convert the label object to string. */
+PHP_METHOD(labelObj, convertToString)
+{
+  zval *zobj = getThis();
+  php_label_object *php_label;
+  char *value = NULL;
+
+  PHP_MAPSCRIPT_ERROR_HANDLING(TRUE);
+  if (zend_parse_parameters_none() == FAILURE) {
+    PHP_MAPSCRIPT_RESTORE_ERRORS(TRUE);
+    return;
+  }
+  PHP_MAPSCRIPT_RESTORE_ERRORS(TRUE);
+
+  php_label = (php_label_object *) zend_object_store_get_object(zobj TSRMLS_CC);
+
+  value =  labelObj_convertToString(php_label->label);
+
+  if (value == NULL)
+    RETURN_STRING("", 1);
+
+  RETVAL_STRING(value, 1);
+  free(value);
+}
+/* }}} */
+
 /* {{{ proto int label.setbinding(const bindingid, string value)
    Set the attribute binding for a specfiled label property. Returns MS_SUCCESS on success. */
 PHP_METHOD(labelObj, setBinding)
 {
   zval *zobj = getThis();
   char *value;
-  long value_len;
+  long value_len = 0;
   long bindingId;
   php_label_object *php_label;
 
@@ -680,6 +707,7 @@ zend_function_entry label_functions[] = {
   PHP_ME(labelObj, __set, label___set_args, ZEND_ACC_PUBLIC)
   PHP_MALIAS(labelObj, set, __set, NULL, ZEND_ACC_PUBLIC)
   PHP_ME(labelObj, updateFromString, label_updateFromString_args, ZEND_ACC_PUBLIC)
+  PHP_ME(labelObj, convertToString, NULL, ZEND_ACC_PUBLIC)
   PHP_ME(labelObj, setBinding, label_setBinding_args, ZEND_ACC_PUBLIC)
   PHP_ME(labelObj, getBinding, label_getBinding_args, ZEND_ACC_PUBLIC)
   PHP_ME(labelObj, removeBinding, label_removeBinding_args, ZEND_ACC_PUBLIC)
