@@ -195,6 +195,10 @@ ZEND_BEGIN_ARG_INFO_EX(layer_getShape_args, 0, 0, 1)
 ZEND_ARG_OBJ_INFO(0, record, resultObj, 0)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(layer_setGeomTransform_args, 0, 0, 1)
+ZEND_ARG_INFO(0, transform)
+ZEND_END_ARG_INFO()
+
 /* {{{ proto void __construct(mapObj map [, layerObj layer])
    Create a new layerObj instance. */
 PHP_METHOD(layerObj, __construct)
@@ -252,7 +256,7 @@ PHP_METHOD(layerObj, __construct)
 PHP_METHOD(layerObj, __get)
 {
   char *property;
-  long property_len;
+  long property_len = 0;
   zval *zobj = getThis();
   php_layer_object *php_layer;
 
@@ -328,7 +332,7 @@ PHP_METHOD(layerObj, __get)
 PHP_METHOD(layerObj, __set)
 {
   char *property;
-  long property_len;
+  long property_len = 0;
   zval *value;
   zval *zobj = getThis();
   php_layer_object *php_layer;
@@ -633,9 +637,9 @@ PHP_METHOD(layerObj, queryByAttributes)
 {
   zval *zobj = getThis();
   char *item;
-  long item_len;
+  long item_len = 0;
   char *string;
-  long string_len;
+  long string_len = 0;
   long mode;
   int status = MS_FAILURE;
   php_layer_object *php_layer;
@@ -674,7 +678,7 @@ PHP_METHOD(layerObj, queryByFilter)
 {
   zval *zobj = getThis();
   char *string;
-  long string_len;
+  long string_len = 0;
   int status = MS_FAILURE;
   php_layer_object *php_layer;
   php_map_object *php_map;
@@ -746,7 +750,7 @@ PHP_METHOD(layerObj, updateFromString)
 {
   zval *zobj = getThis();
   char *snippet;
-  long snippet_len;
+  long snippet_len = 0;
   int status = MS_FAILURE;
   php_layer_object *php_layer;
 
@@ -769,6 +773,33 @@ PHP_METHOD(layerObj, updateFromString)
 }
 /* }}} */
 
+/* {{{ proto string convertToString()
+   Convert the layer object to string. */
+PHP_METHOD(layerObj, convertToString)
+{
+  zval *zobj = getThis();
+  php_layer_object *php_layer;
+  char *value = NULL;
+
+  PHP_MAPSCRIPT_ERROR_HANDLING(TRUE);
+  if (zend_parse_parameters_none() == FAILURE) {
+    PHP_MAPSCRIPT_RESTORE_ERRORS(TRUE);
+    return;
+  }
+  PHP_MAPSCRIPT_RESTORE_ERRORS(TRUE);
+
+  php_layer = (php_layer_object *) zend_object_store_get_object(zobj TSRMLS_CC);
+
+  value =  layerObj_convertToString(php_layer->layer);
+
+  if (value == NULL)
+    RETURN_STRING("", 1);
+
+  RETVAL_STRING(value, 1);
+  free(value);
+}
+/* }}} */
+
 /* {{{ proto int layer.getClass(int i)
    Returns a classObj from the layer given an index value (0=first class) */
 PHP_METHOD(layerObj, getClass)
@@ -863,7 +894,7 @@ PHP_METHOD(layerObj, setFilter)
 {
   zval *zobj = getThis();
   char *expression;
-  long expression_len;
+  long expression_len = 0;
   int status = MS_FAILURE;
   php_layer_object *php_layer;
 
@@ -919,7 +950,7 @@ PHP_METHOD(layerObj, setProjection)
 {
   zval *zobj = getThis();
   char *projection;
-  long projection_len;
+  long projection_len = 0;
   int status = MS_FAILURE;
   php_layer_object *php_layer;
   php_projection_object *php_projection=NULL;
@@ -981,7 +1012,7 @@ PHP_METHOD(layerObj, setWKTProjection)
 {
   zval *zobj = getThis();
   char *projection;
-  long projection_len;
+  long projection_len = 0;
   int status = MS_FAILURE;
   php_layer_object *php_layer;
   php_projection_object *php_projection=NULL;
@@ -1383,7 +1414,7 @@ PHP_METHOD(layerObj, getWMSFeatureInfoURL)
   zval *zobj = getThis();
   long clickx, clicky, featureCount;
   char *infoFormat = NULL;
-  long infoFormat_len;
+  long infoFormat_len = 0;
   char *value =  NULL;
   php_layer_object *php_layer;
   php_map_object *php_map;
@@ -1455,7 +1486,7 @@ PHP_METHOD(layerObj, setProcessing)
 {
   zval *zobj = getThis();
   char *string = NULL;
-  long string_len;
+  long string_len = 0;
   php_layer_object *php_layer;
 
   PHP_MAPSCRIPT_ERROR_HANDLING(TRUE);
@@ -1487,9 +1518,9 @@ PHP_METHOD(layerObj, setProcessingKey)
 {
   zval *zobj = getThis();
   char *key = NULL;
-  long key_len;
+  long key_len = 0;
   char *value = NULL;
-  long value_len;
+  long value_len = 0;
   php_layer_object *php_layer;
 
   PHP_MAPSCRIPT_ERROR_HANDLING(TRUE);
@@ -1595,9 +1626,9 @@ PHP_METHOD(layerObj, applySLD)
 {
   zval *zobj = getThis();
   char *sldxml;
-  long sldxml_len;
+  long sldxml_len = 0;
   char *namedLayer = NULL;
-  long namedLayer_len;
+  long namedLayer_len = 0;
   int status = MS_FAILURE;
   php_layer_object *php_layer;
 
@@ -1628,9 +1659,9 @@ PHP_METHOD(layerObj, applySLDURL)
 {
   zval *zobj = getThis();
   char *sldurl;
-  long sldurl_len;
+  long sldurl_len = 0;
   char *namedLayer = NULL;
-  long namedLayer_len;
+  long namedLayer_len = 0;
   int status = MS_FAILURE;
   php_layer_object *php_layer;
 
@@ -1804,7 +1835,7 @@ PHP_METHOD(layerObj, setConnectionType)
   zval *zobj = getThis();
   long type;
   char *plugin = "";
-  long plugin_len;
+  long plugin_len = 0;
   int status = MS_FAILURE;
   php_layer_object *php_layer;
 
@@ -2016,6 +2047,61 @@ PHP_METHOD(layerObj, getShape)
 }
 /* }}} */
 
+/* {{{ proto int layer.getGeomTransform()
+   return the geometry transform expression */
+PHP_METHOD(layerObj, getGeomTransform)
+{
+  zval *zobj = getThis();
+  php_layer_object *php_layer;
+
+  PHP_MAPSCRIPT_ERROR_HANDLING(TRUE);
+  if (zend_parse_parameters_none() == FAILURE) {
+    PHP_MAPSCRIPT_RESTORE_ERRORS(TRUE);
+    return;
+  }
+  PHP_MAPSCRIPT_RESTORE_ERRORS(TRUE);
+
+  php_layer = (php_layer_object *) zend_object_store_get_object(zobj TSRMLS_CC);
+
+  if (php_layer->layer->_geomtransform.type == MS_GEOMTRANSFORM_NONE ||
+      !php_layer->layer->_geomtransform.string)
+    RETURN_STRING("", 1);
+
+  RETURN_STRING(php_layer->layer->_geomtransform.string, 1);
+}
+/* }}} */
+
+/* {{{ proto int layer.setGeomTransform()
+   set the geometry transform expression */
+PHP_METHOD(layerObj, setGeomTransform)
+{
+  zval *zobj = getThis();
+  char *transform;
+  long transform_len = 0;
+  php_layer_object *php_layer;
+
+  PHP_MAPSCRIPT_ERROR_HANDLING(TRUE);
+  if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s",
+                            &transform, &transform_len) == FAILURE) {
+    PHP_MAPSCRIPT_RESTORE_ERRORS(TRUE);
+    return;
+  }
+  PHP_MAPSCRIPT_RESTORE_ERRORS(TRUE);
+
+  php_layer = (php_layer_object *) zend_object_store_get_object(zobj TSRMLS_CC);
+
+  msFree(php_layer->layer->_geomtransform.string);
+  if (transform_len > 0) {
+    php_layer->layer->_geomtransform.string = msStrdup(transform);
+    php_layer->layer->_geomtransform.type = MS_GEOMTRANSFORM_EXPRESSION;
+  }
+  else {
+    php_layer->layer->_geomtransform.type = MS_GEOMTRANSFORM_NONE;
+    php_layer->layer->_geomtransform.string = NULL;    
+  }
+
+  RETURN_LONG(MS_SUCCESS);  
+}
 
 /* {{{ proto void layer.free()
    Free the object */
@@ -2051,6 +2137,7 @@ zend_function_entry layer_functions[] = {
   PHP_ME(layerObj, draw, layer_draw_args, ZEND_ACC_PUBLIC)
   PHP_ME(layerObj, drawQuery, layer_drawQuery_args, ZEND_ACC_PUBLIC)
   PHP_ME(layerObj, updateFromString, layer_updateFromString_args, ZEND_ACC_PUBLIC)
+  PHP_ME(layerObj, convertToString, NULL, ZEND_ACC_PUBLIC)
   PHP_ME(layerObj, getClass, layer_getClass_args, ZEND_ACC_PUBLIC)
   PHP_ME(layerObj, getClassIndex, layer_getClassIndex_args, ZEND_ACC_PUBLIC)
   PHP_ME(layerObj, queryByPoint, layer_queryByPoint_args, ZEND_ACC_PUBLIC)
@@ -2095,6 +2182,8 @@ zend_function_entry layer_functions[] = {
   PHP_ME(layerObj, setConnectionType, layer_setConnectionType_args, ZEND_ACC_PUBLIC)
   PHP_ME(layerObj, getGridIntersectionCoordinates, NULL, ZEND_ACC_PUBLIC)
   PHP_ME(layerObj, getShape, layer_getShape_args, ZEND_ACC_PUBLIC)
+  PHP_ME(layerObj, getGeomTransform, NULL, ZEND_ACC_PUBLIC)
+  PHP_ME(layerObj, setGeomTransform, layer_setGeomTransform_args, ZEND_ACC_PUBLIC)  
   PHP_ME(layerObj, free, NULL, ZEND_ACC_PUBLIC) {
     NULL, NULL, NULL
   }
