@@ -141,7 +141,7 @@ imageObj *mapObj_drawQuery(mapObj* self)
 
 imageObj *mapObj_drawLegend(mapObj* self)
 {
-  return msDrawLegend(self, MS_FALSE);
+  return msDrawLegend(self, MS_FALSE, NULL);
 }
 
 
@@ -459,6 +459,11 @@ int mapObj_scaleExtent(mapObj *self, double zoomfactor, double minscaledenom,
   return msMapScaleExtent(self, zoomfactor, minscaledenom, maxscaledenom);
 }
 
+char *mapObj_convertToString(mapObj *self)
+{
+  return msWriteMapToString(self);
+}
+
 /**********************************************************************
  * class extensions for layerObj, always within the context of a map
  **********************************************************************/
@@ -508,6 +513,11 @@ int layerObj_updateFromString(layerObj *self, char *snippet)
   return msUpdateLayerFromString(self, snippet, MS_FALSE);
 }
 
+char *layerObj_convertToString(layerObj *self)
+{
+  return msWriteLayerToString(self);
+}
+
 int layerObj_open(layerObj *self)
 {
   return msLayerOpen(self);
@@ -894,6 +904,11 @@ int labelObj_updateFromString(labelObj *self, char *snippet)
   return msUpdateLabelFromString(self, snippet, MS_FALSE);
 }
 
+char *labelObj_convertToString(labelObj *self)
+{
+  return msWriteLabelToString(self);
+}
+
 int labelObj_moveStyleUp(labelObj *self, int index)
 {
   return msMoveLabelStyleUp(self, index);
@@ -944,6 +959,11 @@ int legendObj_updateFromString(legendObj *self, char *snippet)
   return msUpdateLegendFromString(self, snippet, MS_FALSE);
 }
 
+char *legendObj_convertToString(legendObj *self)
+{
+  return msWriteLegendToString(self);
+}
+
 /**********************************************************************
  * class extensions for queryMapObj
  **********************************************************************/
@@ -952,6 +972,11 @@ int queryMapObj_updateFromString(queryMapObj *self, char *snippet)
   return msUpdateQueryMapFromString(self, snippet, MS_FALSE);
 }
 
+char *queryMapObj_convertToString(queryMapObj *self)
+{
+  return msWriteQueryMapToString(self);
+}
+
 /**********************************************************************
  * class extensions for referenceMapObj
  **********************************************************************/
@@ -961,6 +986,11 @@ int referenceMapObj_updateFromString(referenceMapObj *self, char *snippet)
   return msUpdateReferenceMapFromString(self, snippet, MS_FALSE);
 }
 
+char *referenceMapObj_convertToString(referenceMapObj *self)
+{
+  return msWriteReferenceMapToString(self);
+}
+
 /**********************************************************************
  * class extensions for scaleBarObj
  **********************************************************************/
@@ -970,6 +1000,11 @@ int scalebarObj_updateFromString(scalebarObj *self, char *snippet)
   return msUpdateScalebarFromString(self, snippet, MS_FALSE);
 }
 
+char *scalebarObj_convertToString(scalebarObj *self)
+{
+  return msWriteScalebarToString(self);
+}
+
 /**********************************************************************
  * class extensions for webObj
  **********************************************************************/
@@ -979,6 +1014,11 @@ int webObj_updateFromString(webObj *self, char *snippet)
   return msUpdateWebFromString(self, snippet, MS_FALSE);
 }
 
+char *webObj_convertToString(webObj *self)
+{
+  return msWriteWebToString(self);
+}
+
 /**********************************************************************
  * class extensions for classObj, always within the context of a layer
  **********************************************************************/
@@ -1026,6 +1066,11 @@ int classObj_updateFromString(classObj *self, char *snippet)
   return msUpdateClassFromString(self, snippet, MS_FALSE);
 }
 
+char *classObj_convertToString(classObj *self)
+{
+  return msWriteClassToString(self);
+}
+
 void  classObj_destroy(classObj *self)
 {
   return; /* do nothing, map deconstrutor takes care of it all */
@@ -1063,12 +1108,12 @@ int classObj_drawLegendIcon(classObj *self, mapObj *map, layerObj *layer, int wi
 #ifdef USE_GD
   msClearLayerPenValues(layer); // just in case the mapfile has already been processed
 #endif
-  return msDrawLegendIcon(map, layer, self, width, height, dstImg, dstX, dstY);
+  return msDrawLegendIcon(map, layer, self, width, height, dstImg, dstX, dstY, MS_TRUE, NULL);
 }
 
 imageObj *classObj_createLegendIcon(classObj *self, mapObj *map, layerObj *layer, int width, int height)
 {
-  return msCreateLegendIcon(map, layer, self, width, height);
+  return msCreateLegendIcon(map, layer, self, width, height, MS_TRUE);
 }
 
 
@@ -1729,6 +1774,11 @@ int styleObj_updateFromString(styleObj *self, char *snippet)
   return msUpdateStyleFromString(self, snippet, MS_FALSE);
 }
 
+char *styleObj_convertToString(styleObj *self)
+{
+  return msWriteStyleToString(self);
+}
+
 int styleObj_setSymbolByName(styleObj *self, mapObj *map, char* pszSymbolName)
 {
   self->symbol = msGetSymbolIndex(&map->symbolset, pszSymbolName, MS_TRUE);
@@ -1905,6 +1955,11 @@ int clusterObj_updateFromString(clusterObj *self, char *snippet)
   return msUpdateClusterFromString(self, snippet);
 }
 
+char *clusterObj_convertToString(clusterObj *self)
+{
+  return msWriteClusterToString(self);
+}
+
 int clusterObj_setGroup(clusterObj *self, char *string)
 {
   if (!string || strlen(string) == 0) {
