@@ -131,7 +131,7 @@ PHP_METHOD(styleObj, __construct)
 PHP_METHOD(styleObj, __get)
 {
   char *property;
-  long property_len;
+  long property_len = 0;
   zval *zobj = getThis();
   php_style_object *php_style;
 
@@ -189,7 +189,7 @@ PHP_METHOD(styleObj, __get)
 PHP_METHOD(styleObj, __set)
 {
   char *property;
-  long property_len;
+  long property_len = 0;
   zval *value;
   zval *zobj = getThis();
   php_style_object *php_style;
@@ -304,7 +304,7 @@ PHP_METHOD(styleObj, updateFromString)
 {
   zval *zobj = getThis();
   char *snippet;
-  long snippet_len;
+  long snippet_len = 0;
   int status = MS_FAILURE;
   zval *retval;
   zval property_name, value;
@@ -338,13 +338,40 @@ PHP_METHOD(styleObj, updateFromString)
 }
 /* }}} */
 
+/* {{{ proto string convertToString()
+   Convert the style object to string. */
+PHP_METHOD(styleObj, convertToString)
+{
+  zval *zobj = getThis();
+  php_style_object *php_style;
+  char *value = NULL;
+
+  PHP_MAPSCRIPT_ERROR_HANDLING(TRUE);
+  if (zend_parse_parameters_none() == FAILURE) {
+    PHP_MAPSCRIPT_RESTORE_ERRORS(TRUE);
+    return;
+  }
+  PHP_MAPSCRIPT_RESTORE_ERRORS(TRUE);
+
+  php_style = (php_style_object *) zend_object_store_get_object(zobj TSRMLS_CC);
+
+  value =  styleObj_convertToString(php_style->style);
+
+  if (value == NULL)
+    RETURN_STRING("", 1);
+
+  RETVAL_STRING(value, 1);
+  free(value);
+}
+/* }}} */
+
 /* {{{ proto int style.setbinding(const bindingid, string value)
    Set the attribute binding for a specfiled style property. Returns MS_SUCCESS on success. */
 PHP_METHOD(styleObj, setBinding)
 {
   zval *zobj = getThis();
   char *value;
-  long value_len;
+  long value_len = 0;
   long bindingId;
   php_style_object *php_style;
 
@@ -503,7 +530,7 @@ PHP_METHOD(styleObj, setGeomTransform)
 {
   zval *zobj = getThis();
   char *transform;
-  long transform_len;
+  long transform_len = 0;
   php_style_object *php_style;
 
   PHP_MAPSCRIPT_ERROR_HANDLING(TRUE);
@@ -599,6 +626,7 @@ zend_function_entry style_functions[] = {
   PHP_ME(styleObj, __set, style___set_args, ZEND_ACC_PUBLIC)
   PHP_MALIAS(styleObj, set, __set, NULL, ZEND_ACC_PUBLIC)
   PHP_ME(styleObj, updateFromString, style_updateFromString_args, ZEND_ACC_PUBLIC)
+  PHP_ME(styleObj, convertToString, NULL, ZEND_ACC_PUBLIC)
   PHP_ME(styleObj, setBinding, style_setBinding_args, ZEND_ACC_PUBLIC)
   PHP_ME(styleObj, getBinding, style_getBinding_args, ZEND_ACC_PUBLIC)
   PHP_ME(styleObj, removeBinding, style_removeBinding_args, ZEND_ACC_PUBLIC)
