@@ -1,4 +1,4 @@
-# $Id: setup.py 10457 2010-08-14 16:41:44Z hobu $
+# $Id$
 #
 # setup.py file for MapScript
 #
@@ -50,7 +50,7 @@ def unique(list):
 # ---------------------------------------------------------------------------
 
 include_dirs = ['../..']
-library_dirs = ['../../']
+library_dirs = ['../../.libs']
 libraries = ['mapserver']
 
 extra_link_args = []
@@ -131,6 +131,8 @@ class ms_ext(build_ext):
     ])
 
     def initialize_options(self):
+        print "LD_RUN_PATH set"
+        os.environ["LD_RUN_PATH"] = os.getcwd()+"/../../.libs"
         build_ext.initialize_options(self)
         self.gdaldir = None
         self.mapserver_config = self.MAPSERVER_CONFIG
@@ -273,4 +275,4 @@ else:
            py_modules = py_modules,
            url=url,
            cmdclass={'build_ext':ms_ext},
-           ext_modules = ext_modules )    
\ No newline at end of file
+           ext_modules = ext_modules )    
