@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapsearch.c 10535 2010-09-29 18:18:31Z warmerdam $
+ * $Id$
  *
  * Project:  MapServer
  * Purpose:  Various geospatial search operations.
@@ -8,7 +8,7 @@
  * Notes: For information on point in polygon function please see:
  *
  *   http://www.ecse.rpi.edu/Homepages/wrf/Research/Short_Notes/pnpoly.html
- * 
+ *
  * The appropriate copyright notice accompanies the funtion definition.
  *
  ******************************************************************************
@@ -21,7 +21,7 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  *
- * The above copyright notice and this permission notice shall be included in 
+ * The above copyright notice and this permission notice shall be included in
  * all copies of this Software or works derived from this Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
@@ -35,7 +35,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapsearch.c 10535 2010-09-29 18:18:31Z warmerdam $")
+
 
 #define LASTVERT(v,n)  ((v) == 0 ? n-2 : v-1)
 #define NEXTVERT(v,n)  ((v) == n-2 ? 0 : v+1)
@@ -55,24 +55,24 @@ int msRectOverlap(rectObj *a, rectObj *b)
 /*
 ** Computes the intersection of two rectangles, updating the first
 ** to be only the intersection of the two.  Returns MS_FALSE if
-** the intersection is empty. 
+** the intersection is empty.
 */
 int msRectIntersect( rectObj *a, const rectObj *b )
 {
-    if( a->maxx > b->maxx )
-        a->maxx = b->maxx;
-    if( a->minx < b->minx )
-        a->minx = b->minx;
-    if( a->maxy > b->maxy )
-        a->maxy = b->maxy;
-    if( a->miny < b->miny )
-        a->miny = b->miny;
+  if( a->maxx > b->maxx )
+    a->maxx = b->maxx;
+  if( a->minx < b->minx )
+    a->minx = b->minx;
+  if( a->maxy > b->maxy )
+    a->maxy = b->maxy;
+  if( a->miny < b->miny )
+    a->miny = b->miny;
 
-    if( a->maxx < a->minx || b->maxx < b->minx )
-        return MS_FALSE;
-    else 
-        return MS_TRUE;
-}        
+  if( a->maxx < a->minx || b->maxx < b->minx )
+    return MS_FALSE;
+  else
+    return MS_TRUE;
+}
 
 /*
 ** Returns MS_TRUE if rectangle a is contained in rectangle b
@@ -82,7 +82,7 @@ int msRectContained(rectObj *a, rectObj *b)
   if(a->minx >= b->minx && a->maxx <= b->maxx)
     if(a->miny >= b->miny && a->maxy <= b->maxy)
       return(MS_TRUE);
-  return(MS_FALSE);  
+  return(MS_FALSE);
 }
 
 /*
@@ -125,37 +125,36 @@ int msPolygonDirection(lineObj *c)
   lv = LASTVERT(v,c->numpoints);
   nv = NEXTVERT(v,c->numpoints);
 
-  area = c->point[lv].x*c->point[v].y - c->point[lv].y*c->point[v].x + c->point[lv].y*c->point[nv].x - c->point[lv].x*c->point[nv].y + c->point[v].x*c->point[nv].y - c->point[nv].x*c->point[v].y;   
+  area = c->point[lv].x*c->point[v].y - c->point[lv].y*c->point[v].x + c->point[lv].y*c->point[nv].x - c->point[lv].x*c->point[nv].y + c->point[v].x*c->point[nv].y - c->point[nv].x*c->point[v].y;
   if(area > 0)
-    return(1); /* counter clockwise orientation */ 
-  else 
-    if(area < 0) /* clockwise orientation */
-      return(-1);
-    else
-      return(0); /* shouldn't happen unless the polygon is self intersecting */
+    return(1); /* counter clockwise orientation */
+  else if(area < 0) /* clockwise orientation */
+    return(-1);
+  else
+    return(0); /* shouldn't happen unless the polygon is self intersecting */
 }
 
-/*                                                                                                                         
-** Copyright (c) 1970-2003, Wm. Randolph Franklin                                                                          
-**                                                                                                                         
-** Permission is hereby granted, free of charge, to any person obtaining a copy of this software and                       
-** associated documentation files (the "Software"), to deal in the Software without restriction, including                 
-** without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell                 
-** copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the                
-** following conditions:                                                                                                   
-**                                                                                                                         
-** 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the               
-**    following disclaimers.                                                                                               
-** 2. Redistributions in binary form must reproduce the above copyright notice in the documentation and/or                 
-**    other materials provided with the distribution.                                                                      
-** 3. The name of W. Randolph Franklin may not be used to endorse or promote products derived from this                    
-**    Software without specific prior written permission.                                                                  
-**                                                                                                                         
-** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT                   
-** LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN                  
-** NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,                    
-** WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE                     
-** SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                                                  
+/*
+** Copyright (c) 1970-2003, Wm. Randolph Franklin
+**
+** Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
+** associated documentation files (the "Software"), to deal in the Software without restriction, including
+** without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+** copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the
+** following conditions:
+**
+** 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
+**    following disclaimers.
+** 2. Redistributions in binary form must reproduce the above copyright notice in the documentation and/or
+**    other materials provided with the distribution.
+** 3. The name of W. Randolph Franklin may not be used to endorse or promote products derived from this
+**    Software without specific prior written permission.
+**
+** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
+** LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+** NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+** WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+** SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 int msPointInPolygon(pointObj *p, lineObj *c)
 {
@@ -174,25 +173,26 @@ int msPointInPolygon(pointObj *p, lineObj *c)
 ** cases. In due time... -SDL-
 */
 
-int msIntersectSegments(pointObj *a, pointObj *b, pointObj *c, pointObj *d) { /* from comp.graphics.alogorithms FAQ */
+int msIntersectSegments(pointObj *a, pointObj *b, pointObj *c, pointObj *d)   /* from comp.graphics.alogorithms FAQ */
+{
 
   double r, s;
   double denominator, numerator;
 
-  numerator = ((a->y-c->y)*(d->x-c->x) - (a->x-c->x)*(d->y-c->y));  
-  denominator = ((b->x-a->x)*(d->y-c->y) - (b->y-a->y)*(d->x-c->x));  
+  numerator = ((a->y-c->y)*(d->x-c->x) - (a->x-c->x)*(d->y-c->y));
+  denominator = ((b->x-a->x)*(d->y-c->y) - (b->y-a->y)*(d->x-c->x));
 
   if((denominator == 0) && (numerator == 0)) { /* lines are coincident, intersection is a line segement if it exists */
     if(a->y == c->y) { /* coincident horizontally, check x's */
       if(((a->x >= MS_MIN(c->x,d->x)) && (a->x <= MS_MAX(c->x,d->x))) || ((b->x >= MS_MIN(c->x,d->x)) && (b->x <= MS_MAX(c->x,d->x))))
-  return(MS_TRUE);
+        return(MS_TRUE);
       else
-  return(MS_FALSE);
+        return(MS_FALSE);
     } else { /* test for y's will work fine for remaining cases */
       if(((a->y >= MS_MIN(c->y,d->y)) && (a->y <= MS_MAX(c->y,d->y))) || ((b->y >= MS_MIN(c->y,d->y)) && (b->y <= MS_MAX(c->y,d->y))))
-  return(MS_TRUE);
+        return(MS_TRUE);
       else
-  return(MS_FALSE);
+        return(MS_FALSE);
     }
   }
 
@@ -218,7 +218,8 @@ int msIntersectSegments(pointObj *a, pointObj *b, pointObj *c, pointObj *d) { /*
 ** point falls in. If odd the point is in the polygon, if 0 or even
 ** then the point is in a hole or completely outside.
 */
-int msIntersectPointPolygon(pointObj *point, shapeObj *poly) {
+int msIntersectPointPolygon(pointObj *point, shapeObj *poly)
+{
   int i;
   int status=MS_FALSE;
 
@@ -227,10 +228,11 @@ int msIntersectPointPolygon(pointObj *point, shapeObj *poly) {
       status = !status;
   }
 
-  return(status);  
+  return(status);
 }
 
-int msIntersectMultipointPolygon(shapeObj *multipoint, shapeObj *poly) {
+int msIntersectMultipointPolygon(shapeObj *multipoint, shapeObj *poly)
+{
   int i,j;
 
   /* The change to loop through all the lines has been made for ticket
@@ -239,15 +241,16 @@ int msIntersectMultipointPolygon(shapeObj *multipoint, shapeObj *poly) {
   for(i=0; i<multipoint->numlines; i++ ) {
     lineObj points = multipoint->line[i];
     for(j=0; j<points.numpoints; j++) {
-          if(msIntersectPointPolygon(&(points.point[j]), poly) == MS_TRUE)
+      if(msIntersectPointPolygon(&(points.point[j]), poly) == MS_TRUE)
         return(MS_TRUE);
     }
   }
-    
+
   return(MS_FALSE);
 }
 
-int msIntersectPolylines(shapeObj *line1, shapeObj *line2) {
+int msIntersectPolylines(shapeObj *line1, shapeObj *line2)
+{
   int c1,v1,c2,v2;
 
   for(c1=0; c1<line1->numlines; c1++)
@@ -255,13 +258,14 @@ int msIntersectPolylines(shapeObj *line1, shapeObj *line2) {
       for(c2=0; c2<line2->numlines; c2++)
         for(v2=1; v2<line2->line[c2].numpoints; v2++)
           if(msIntersectSegments(&(line1->line[c1].point[v1-1]), &(line1->line[c1].point[v1]),
-              &(line2->line[c2].point[v2-1]), &(line2->line[c2].point[v2])) ==  MS_TRUE)
+                                 &(line2->line[c2].point[v2-1]), &(line2->line[c2].point[v2])) ==  MS_TRUE)
             return(MS_TRUE);
 
   return(MS_FALSE);
 }
 
-int msIntersectPolylinePolygon(shapeObj *line, shapeObj *poly) {
+int msIntersectPolylinePolygon(shapeObj *line, shapeObj *poly)
+{
   int i;
 
   /* STEP 1: polygon might competely contain the polyline or one of it's parts (only need to check one point from each part) */
@@ -277,7 +281,8 @@ int msIntersectPolylinePolygon(shapeObj *line, shapeObj *poly) {
   return(MS_FALSE);
 }
 
-int msIntersectPolygons(shapeObj *p1, shapeObj *p2) {
+int msIntersectPolygons(shapeObj *p1, shapeObj *p2)
+{
   int i;
 
   /* STEP 1: polygon 1 completely contains 2 (only need to check one point from each part) */
@@ -324,7 +329,7 @@ double msDistancePointToPoint(pointObj *a, pointObj *b)
 double msSquareDistancePointToPoint(pointObj *a, pointObj *b)
 {
   double dx, dy;
-  
+
   dx = a->x - b->x;
   dy = a->y - b->y;
 
@@ -333,7 +338,7 @@ double msSquareDistancePointToPoint(pointObj *a, pointObj *b)
 
 double msDistancePointToSegment(pointObj *p, pointObj *a, pointObj *b)
 {
-	return (sqrt(msSquareDistancePointToSegment(p, a, b)));
+  return (sqrt(msSquareDistancePointToSegment(p, a, b)));
 }
 
 double msSquareDistancePointToSegment(pointObj *p, pointObj *a, pointObj *b)
@@ -375,11 +380,11 @@ double msSquareDistancePointToSegment(pointObj *p, pointObj *a, pointObj *b)
 /* liable for any real or imagined damage resulting from its use. */
 /* Users of this code must verify correctness for their application. */
 
-double msDistanceSegmentToSegment(pointObj *pa, pointObj *pb, pointObj *pc, pointObj *pd) 
+double msDistanceSegmentToSegment(pointObj *pa, pointObj *pb, pointObj *pc, pointObj *pd)
 {
   vectorObj dP;
   vectorObj u, v, w;
-  double a, b, c, d, e; 
+  double a, b, c, d, e;
   double D;
   double sc, sN, sD; /* N=numerator, D=demoninator */
   double tc, tN, tD;
@@ -395,9 +400,9 @@ double msDistanceSegmentToSegment(pointObj *pa, pointObj *pb, pointObj *pc, poin
   u.x = pb->x - pa->x; /* u = pb - pa */
   u.y = pb->y - pa->y;
   v.x = pd->x - pc->x; /* v = pd - pc  */
-  v.y = pd->y - pc->y; 
+  v.y = pd->y - pc->y;
   w.x = pa->x - pc->x; /* w = pa - pc */
-  w.y = pa->y - pc->y; 
+  w.y = pa->y - pc->y;
 
   a = dot(u,u);
   b = dot(u,v);
@@ -408,7 +413,7 @@ double msDistanceSegmentToSegment(pointObj *pa, pointObj *pb, pointObj *pc, poin
   D = a*c - b*b;
   sc = sN = sD = D;
   tc = tN = tD = D;
-  
+
   /* compute the line parameters of the two closest points */
   if(D < SMALL_NUMBER) { /* lines are parallel or almost parallel */
     sN = 0.0;
@@ -479,36 +484,36 @@ double msSquareDistancePointToShape(pointObj *point, shapeObj *shape)
   double dist, minDist=-1;
 
   switch(shape->type) {
-  case(MS_SHAPE_POINT):
-    for(j=0;j<shape->numlines;j++) {
-      for(i=0; i<shape->line[j].numpoints; i++) {
-        dist = msSquareDistancePointToPoint(point, &(shape->line[j].point[i]));
-        if((dist < minDist) || (minDist < 0)) minDist = dist;
+    case(MS_SHAPE_POINT):
+      for(j=0; j<shape->numlines; j++) {
+        for(i=0; i<shape->line[j].numpoints; i++) {
+          dist = msSquareDistancePointToPoint(point, &(shape->line[j].point[i]));
+          if((dist < minDist) || (minDist < 0)) minDist = dist;
+        }
       }
-    }
-    break;
-  case(MS_SHAPE_LINE):
-    for(j=0;j<shape->numlines;j++) {
-      for(i=1; i<shape->line[j].numpoints; i++) {
-        dist = msSquareDistancePointToSegment(point, &(shape->line[j].point[i-1]), &(shape->line[j].point[i]));
-        if((dist < minDist) || (minDist < 0)) minDist = dist;
-      }
-    }
-    break;
-  case(MS_SHAPE_POLYGON):
-    if(msIntersectPointPolygon(point, shape))
-      minDist = 0; /* point is IN the shape */
-    else { /* treat shape just like a line */
-      for(j=0;j<shape->numlines;j++) {
+      break;
+    case(MS_SHAPE_LINE):
+      for(j=0; j<shape->numlines; j++) {
         for(i=1; i<shape->line[j].numpoints; i++) {
           dist = msSquareDistancePointToSegment(point, &(shape->line[j].point[i-1]), &(shape->line[j].point[i]));
           if((dist < minDist) || (minDist < 0)) minDist = dist;
         }
       }
-    }
-    break;
-  default:
-    break;
+      break;
+    case(MS_SHAPE_POLYGON):
+      if(msIntersectPointPolygon(point, shape))
+        minDist = 0; /* point is IN the shape */
+      else { /* treat shape just like a line */
+        for(j=0; j<shape->numlines; j++) {
+          for(i=1; i<shape->line[j].numpoints; i++) {
+            dist = msSquareDistancePointToSegment(point, &(shape->line[j].point[i-1]), &(shape->line[j].point[i]));
+            if((dist < minDist) || (minDist < 0)) minDist = dist;
+          }
+        }
+      }
+      break;
+    default:
+      break;
   }
 
   return(minDist);
@@ -520,143 +525,143 @@ double msDistanceShapeToShape(shapeObj *shape1, shapeObj *shape2)
   double dist, minDist=-1;
 
   switch(shape1->type) {
-  case(MS_SHAPE_POINT): /* shape1 */
-    for(i=0;i<shape1->numlines;i++) {
-      for(j=0; j<shape1->line[i].numpoints; j++) {
-        dist = msSquareDistancePointToShape(&(shape1->line[i].point[j]), shape2);
-        if((dist < minDist) || (minDist < 0)) 
-    minDist = dist;
-      }
-    }
-    minDist = sqrt(minDist);
-    break;
-  case(MS_SHAPE_LINE): /* shape1 */
-    switch(shape2->type) {
-    case(MS_SHAPE_POINT):
-      for(i=0;i<shape2->numlines;i++) {
-        for(j=0; j<shape2->line[i].numpoints; j++) {
-          dist = msSquareDistancePointToShape(&(shape2->line[i].point[j]), shape1);
-          if((dist < minDist) || (minDist < 0)) 
-      minDist = dist;
+    case(MS_SHAPE_POINT): /* shape1 */
+      for(i=0; i<shape1->numlines; i++) {
+        for(j=0; j<shape1->line[i].numpoints; j++) {
+          dist = msSquareDistancePointToShape(&(shape1->line[i].point[j]), shape2);
+          if((dist < minDist) || (minDist < 0))
+            minDist = dist;
         }
       }
       minDist = sqrt(minDist);
       break;
-    case(MS_SHAPE_LINE):
-      for(i=0;i<shape1->numlines;i++) {
-        for(j=1; j<shape1->line[i].numpoints; j++) {
-          for(k=0;k<shape2->numlines;k++) {
-            for(l=1; l<shape2->line[k].numpoints; l++) {
-              /* check intersection (i.e. dist=0) */
-        if(msIntersectSegments(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l])) == MS_TRUE) 
-                return(0);
+    case(MS_SHAPE_LINE): /* shape1 */
+      switch(shape2->type) {
+        case(MS_SHAPE_POINT):
+          for(i=0; i<shape2->numlines; i++) {
+            for(j=0; j<shape2->line[i].numpoints; j++) {
+              dist = msSquareDistancePointToShape(&(shape2->line[i].point[j]), shape1);
+              if((dist < minDist) || (minDist < 0))
+                minDist = dist;
+            }
+          }
+          minDist = sqrt(minDist);
+          break;
+        case(MS_SHAPE_LINE):
+          for(i=0; i<shape1->numlines; i++) {
+            for(j=1; j<shape1->line[i].numpoints; j++) {
+              for(k=0; k<shape2->numlines; k++) {
+                for(l=1; l<shape2->line[k].numpoints; l++) {
+                  /* check intersection (i.e. dist=0) */
+                  if(msIntersectSegments(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l])) == MS_TRUE)
+                    return(0);
 
-        /* no intersection, compute distance */
-        dist = msDistanceSegmentToSegment(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l]));
-        if((dist < minDist) || (minDist < 0)) 
-    minDist = dist;
-      }
-    }
-  }
-      }
-      break;    
-    case(MS_SHAPE_POLYGON):
-      /* shape2 (the polygon) could contain shape1 or one of it's parts       */
-      for(i=0; i<shape1->numlines; i++) {
-        if(msIntersectPointPolygon(&(shape1->line[0].point[0]), shape2) == MS_TRUE) /* this considers holes and multiple parts */
-          return(0);
-      }
-      
-      /* check segment intersection and, if necessary, distance between segments */
-      for(i=0;i<shape1->numlines;i++) {
-        for(j=1; j<shape1->line[i].numpoints; j++) {
-          for(k=0;k<shape2->numlines;k++) {
-            for(l=1; l<shape2->line[k].numpoints; l++) {
-        /* check intersection (i.e. dist=0) */
-        if(msIntersectSegments(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l])) == MS_TRUE) 
-                return(0);
+                  /* no intersection, compute distance */
+                  dist = msDistanceSegmentToSegment(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l]));
+                  if((dist < minDist) || (minDist < 0))
+                    minDist = dist;
+                }
+              }
+            }
+          }
+          break;
+        case(MS_SHAPE_POLYGON):
+          /* shape2 (the polygon) could contain shape1 or one of it's parts       */
+          for(i=0; i<shape1->numlines; i++) {
+            if(msIntersectPointPolygon(&(shape1->line[0].point[0]), shape2) == MS_TRUE) /* this considers holes and multiple parts */
+              return(0);
+          }
 
-        /* no intersection, compute distance */
-        dist = msDistanceSegmentToSegment(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l]));
-        if((dist < minDist) || (minDist < 0)) 
-    minDist = dist;
-      }
-    }
-  }
+          /* check segment intersection and, if necessary, distance between segments */
+          for(i=0; i<shape1->numlines; i++) {
+            for(j=1; j<shape1->line[i].numpoints; j++) {
+              for(k=0; k<shape2->numlines; k++) {
+                for(l=1; l<shape2->line[k].numpoints; l++) {
+                  /* check intersection (i.e. dist=0) */
+                  if(msIntersectSegments(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l])) == MS_TRUE)
+                    return(0);
+
+                  /* no intersection, compute distance */
+                  dist = msDistanceSegmentToSegment(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l]));
+                  if((dist < minDist) || (minDist < 0))
+                    minDist = dist;
+                }
+              }
+            }
+          }
+          break;
       }
       break;
-    }
-    break;
-  case(MS_SHAPE_POLYGON): /* shape1 */
-    switch(shape2->type) {
-    case(MS_SHAPE_POINT):
-      for(i=0;i<shape2->numlines;i++) {
-        for(j=0; j<shape2->line[i].numpoints; j++) {
-          dist = msSquareDistancePointToShape(&(shape2->line[i].point[j]), shape1);
-          if((dist < minDist) || (minDist < 0)) 
-      minDist = dist;
-        }
-      }
-      minDist = sqrt(minDist);
-      break;
-    case(MS_SHAPE_LINE):
-      /* shape1 (the polygon) could contain shape2 or one of it's parts       */
-      for(i=0; i<shape2->numlines; i++) {
-        if(msIntersectPointPolygon(&(shape2->line[i].point[0]), shape1) == MS_TRUE) /* this considers holes and multiple parts */
-          return(0);
-      }
-      
-      /* check segment intersection and, if necessary, distance between segments */
-      for(i=0;i<shape1->numlines;i++) {      
-        for(j=1; j<shape1->line[i].numpoints; j++) {
-          for(k=0;k<shape2->numlines;k++) {
-            for(l=1; l<shape2->line[k].numpoints; l++) {
-        /* check intersection (i.e. dist=0) */
-        if(msIntersectSegments(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l])) == MS_TRUE) 
-                return(0);
+    case(MS_SHAPE_POLYGON): /* shape1 */
+      switch(shape2->type) {
+        case(MS_SHAPE_POINT):
+          for(i=0; i<shape2->numlines; i++) {
+            for(j=0; j<shape2->line[i].numpoints; j++) {
+              dist = msSquareDistancePointToShape(&(shape2->line[i].point[j]), shape1);
+              if((dist < minDist) || (minDist < 0))
+                minDist = dist;
+            }
+          }
+          minDist = sqrt(minDist);
+          break;
+        case(MS_SHAPE_LINE):
+          /* shape1 (the polygon) could contain shape2 or one of it's parts       */
+          for(i=0; i<shape2->numlines; i++) {
+            if(msIntersectPointPolygon(&(shape2->line[i].point[0]), shape1) == MS_TRUE) /* this considers holes and multiple parts */
+              return(0);
+          }
 
-        /* no intersection, compute distance */
-        dist = msDistanceSegmentToSegment(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l]));
-        if((dist < minDist) || (minDist < 0)) 
-    minDist = dist;
-      }
-    }
-  }
-      }
-      break; 
-    case(MS_SHAPE_POLYGON): 
-      /* shape1 completely contains shape2 (only need to check one point from each part) */
-      for(i=0; i<shape2->numlines; i++) {
-        if(msIntersectPointPolygon(&(shape2->line[i].point[0]), shape1) == MS_TRUE) /* this considers holes and multiple parts */
-          return(0);
-      }
+          /* check segment intersection and, if necessary, distance between segments */
+          for(i=0; i<shape1->numlines; i++) {
+            for(j=1; j<shape1->line[i].numpoints; j++) {
+              for(k=0; k<shape2->numlines; k++) {
+                for(l=1; l<shape2->line[k].numpoints; l++) {
+                  /* check intersection (i.e. dist=0) */
+                  if(msIntersectSegments(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l])) == MS_TRUE)
+                    return(0);
 
-      /* shape2 completely contains shape1 (only need to check one point from each part) */
-      for(i=0; i<shape1->numlines; i++) {
-        if(msIntersectPointPolygon(&(shape1->line[i].point[0]), shape2) == MS_TRUE) /* this considers holes and multiple parts */
-          return(0);
-      }
+                  /* no intersection, compute distance */
+                  dist = msDistanceSegmentToSegment(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l]));
+                  if((dist < minDist) || (minDist < 0))
+                    minDist = dist;
+                }
+              }
+            }
+          }
+          break;
+        case(MS_SHAPE_POLYGON):
+          /* shape1 completely contains shape2 (only need to check one point from each part) */
+          for(i=0; i<shape2->numlines; i++) {
+            if(msIntersectPointPolygon(&(shape2->line[i].point[0]), shape1) == MS_TRUE) /* this considers holes and multiple parts */
+              return(0);
+          }
 
-      /* check segment intersection and, if necessary, distance between segments */
-      for(i=0;i<shape1->numlines;i++) {        
-        for(j=1; j<shape1->line[i].numpoints; j++) {
-          for(k=0;k<shape2->numlines;k++) {
-            for(l=1; l<shape2->line[k].numpoints; l++) {
-        /* check intersection (i.e. dist=0) */
-        if(msIntersectSegments(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l])) == MS_TRUE) 
-                return(0);        
+          /* shape2 completely contains shape1 (only need to check one point from each part) */
+          for(i=0; i<shape1->numlines; i++) {
+            if(msIntersectPointPolygon(&(shape1->line[i].point[0]), shape2) == MS_TRUE) /* this considers holes and multiple parts */
+              return(0);
+          }
 
-        /* no intersection, compute distance */
-        dist = msDistanceSegmentToSegment(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l]));
-        if((dist < minDist) || (minDist < 0)) 
-    minDist = dist;
-      }
-    }
-  }
+          /* check segment intersection and, if necessary, distance between segments */
+          for(i=0; i<shape1->numlines; i++) {
+            for(j=1; j<shape1->line[i].numpoints; j++) {
+              for(k=0; k<shape2->numlines; k++) {
+                for(l=1; l<shape2->line[k].numpoints; l++) {
+                  /* check intersection (i.e. dist=0) */
+                  if(msIntersectSegments(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l])) == MS_TRUE)
+                    return(0);
+
+                  /* no intersection, compute distance */
+                  dist = msDistanceSegmentToSegment(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l]));
+                  if((dist < minDist) || (minDist < 0))
+                    minDist = dist;
+                }
+              }
+            }
+          }
+          break;
       }
       break;
-    }
-    break;
   }
 
   return(minDist);
