@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maptree.h 8741 2009-03-09 21:04:52Z jlacroix $
+ * $Id$
  *
  * Project:  MapServer
  * Purpose:  .qix spatial index declarations.
@@ -15,7 +15,7 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  *
- * The above copyright notice and this permission notice shall be included in 
+ * The above copyright notice and this permission notice shall be included in
  * all copies of this Software or works derived from this Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
@@ -34,41 +34,40 @@
 extern "C" {
 #endif
 
-/* this can be 2 or 4 for binary or quad tree */
+  /* this can be 2 or 4 for binary or quad tree */
 #define MAX_SUBNODES 4
-   
-typedef struct shape_tree_node {
-  /* area covered by this node */
-  rectObj rect;
-  
-  /* list of shapes stored at this node. */
-  ms_int32 numshapes;
-  ms_int32 *ids;
-  
-  int numsubnodes;
-  struct shape_tree_node *subnode[MAX_SUBNODES];
-} treeNodeObj;
 
-typedef struct {
-  ms_int32 numshapes;
-  ms_int32 maxdepth;
-  treeNodeObj *root;
-} treeObj;
+  typedef struct shape_tree_node {
+    /* area covered by this node */
+    rectObj rect;
+
+    /* list of shapes stored at this node. */
+    ms_int32 numshapes;
+    ms_int32 *ids;
+
+    int numsubnodes;
+    struct shape_tree_node *subnode[MAX_SUBNODES];
+  } treeNodeObj;
+
+  typedef struct {
+    ms_int32 numshapes;
+    ms_int32 maxdepth;
+    treeNodeObj *root;
+  } treeObj;
 
 
-typedef struct
-{
+  typedef struct {
     FILE        *fp;
     char        signature[3];
-    char	LSB_order;
+    char  LSB_order;
     char        needswap;
-    char	version;
-    char	flags[3];
+    char  version;
+    char  flags[3];
 
     ms_int32        nShapes;
     ms_int32        nDepth;
-} SHPTreeInfo;
-typedef SHPTreeInfo * SHPTreeHandle;
+  } SHPTreeInfo;
+  typedef SHPTreeInfo * SHPTreeHandle;
 
 #define MS_LSB_ORDER -1
 #define MS_MSB_ORDER -2
@@ -77,21 +76,21 @@ typedef SHPTreeInfo * SHPTreeHandle;
 #define MS_NEW_MSB_ORDER 2
 
 
-MS_DLL_EXPORT SHPTreeHandle msSHPDiskTreeOpen(const char * pszTree, int debug);
-MS_DLL_EXPORT void msSHPDiskTreeClose(SHPTreeHandle disktree);
-MS_DLL_EXPORT treeNodeObj *readTreeNode( SHPTreeHandle disktree );
+  MS_DLL_EXPORT SHPTreeHandle msSHPDiskTreeOpen(const char * pszTree, int debug);
+  MS_DLL_EXPORT void msSHPDiskTreeClose(SHPTreeHandle disktree);
+  MS_DLL_EXPORT treeNodeObj *readTreeNode( SHPTreeHandle disktree );
 
-MS_DLL_EXPORT treeObj *msCreateTree(shapefileObj *shapefile, int maxdepth);
-MS_DLL_EXPORT void msTreeTrim(treeObj *tree);
-MS_DLL_EXPORT void msDestroyTree(treeObj *tree);
+  MS_DLL_EXPORT treeObj *msCreateTree(shapefileObj *shapefile, int maxdepth);
+  MS_DLL_EXPORT void msTreeTrim(treeObj *tree);
+  MS_DLL_EXPORT void msDestroyTree(treeObj *tree);
 
-MS_DLL_EXPORT ms_bitarray msSearchTree(treeObj *tree, rectObj aoi);
-MS_DLL_EXPORT ms_bitarray msSearchDiskTree(char *filename, rectObj aoi, int debug);
+  MS_DLL_EXPORT ms_bitarray msSearchTree(treeObj *tree, rectObj aoi);
+  MS_DLL_EXPORT ms_bitarray msSearchDiskTree(char *filename, rectObj aoi, int debug);
 
-MS_DLL_EXPORT treeObj *msReadTree(char *filename, int debug);
-MS_DLL_EXPORT int msWriteTree(treeObj *tree, char *filename, int LSB_order);
+  MS_DLL_EXPORT treeObj *msReadTree(char *filename, int debug);
+  MS_DLL_EXPORT int msWriteTree(treeObj *tree, char *filename, int LSB_order);
 
-MS_DLL_EXPORT void msFilterTreeSearch(shapefileObj *shp, ms_bitarray status, rectObj search_rect);
+  MS_DLL_EXPORT void msFilterTreeSearch(shapefileObj *shp, ms_bitarray status, rectObj search_rect);
 
 #ifdef __cplusplus
 }
